### ms-tsts_legacy.idl
"""
RPC definitions for the following interfaces:
- IcaApi (v1.0): 5ca4a760-ebb1-11cf-8611-00a0245420ed
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRInt3264Field,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedLongField,
    register_dcerpc_interface,
)


class RpcWinStationOpenServer_Request(NDRPacket):
    fields_desc = []


class RpcWinStationOpenServer_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pResult", 0),
        NDRPacketField("phServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class RpcWinStationCloseServer_Request(NDRPacket):
    fields_desc = [NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle)]


class RpcWinStationCloseServer_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class RpcIcaServerPing_Request(NDRPacket):
    fields_desc = [NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle)]


class RpcIcaServerPing_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class RpcWinStationEnumerate_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("pEntries", 0),
        NDRFullPointerField(
            NDRConfStrLenField("pLogonId", "", size_is=lambda pkt: pkt.pByteCount)
        ),
        NDRIntField("pByteCount", None, size_of="pLogonId"),
        NDRIntField("pIndex", 0),
    ]


class RpcWinStationEnumerate_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pResult", 0),
        NDRIntField("pEntries", 0),
        NDRFullPointerField(
            NDRConfStrLenField("pLogonId", "", size_is=lambda pkt: pkt.pByteCount)
        ),
        NDRIntField("pByteCount", None, size_of="pLogonId"),
        NDRIntField("pIndex", 0),
        NDRIntField("status", 0),
    ]


class RpcWinStationRename_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRConfStrLenFieldUtf16(
            "pWinStationNameOld", "", size_is=lambda pkt: pkt.NameOldSize
        ),
        NDRIntField("NameOldSize", None, size_of="pWinStationNameOld"),
        NDRConfStrLenFieldUtf16(
            "pWinStationNameNew", "", size_is=lambda pkt: pkt.NameNewSize
        ),
        NDRIntField("NameNewSize", None, size_of="pWinStationNameNew"),
    ]


class RpcWinStationRename_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class RpcWinStationQueryInformation_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("LogonId", 0),
        NDRIntField("WinStationInformationClass", 0),
        NDRFullPointerField(
            NDRConfStrLenField(
                "pWinStationInformation",
                "",
                size_is=lambda pkt: pkt.WinStationInformationLength,
            )
        ),
        NDRIntField(
            "WinStationInformationLength", None, size_of="pWinStationInformation"
        ),
    ]


class RpcWinStationQueryInformation_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pResult", 0),
        NDRFullPointerField(
            NDRConfStrLenField(
                "pWinStationInformation",
                "",
                size_is=lambda pkt: pkt.WinStationInformationLength,
            )
        ),
        NDRIntField("pReturnLength", 0),
        NDRIntField("status", 0),
    ]


class RpcWinStationSetInformation_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("LogonId", 0),
        NDRIntField("WinStationInformationClass", 0),
        NDRFullPointerField(
            NDRConfStrLenField(
                "pWinStationInformation",
                "",
                size_is=lambda pkt: pkt.WinStationInformationLength,
            )
        ),
        NDRIntField(
            "WinStationInformationLength", None, size_of="pWinStationInformation"
        ),
    ]


class RpcWinStationSetInformation_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pResult", 0),
        NDRFullPointerField(
            NDRConfStrLenField(
                "pWinStationInformation",
                "",
                size_is=lambda pkt: pkt.WinStationInformationLength,
            )
        ),
        NDRIntField("status", 0),
    ]


class RpcWinStationSendMessage_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("LogonId", 0),
        NDRConfStrLenFieldUtf16("pTitle", "", size_is=lambda pkt: pkt.TitleLength),
        NDRIntField("TitleLength", None, size_of="pTitle"),
        NDRConfStrLenFieldUtf16("pMessage", "", size_is=lambda pkt: pkt.MessageLength),
        NDRIntField("MessageLength", None, size_of="pMessage"),
        NDRIntField("Style", 0),
        NDRIntField("Timeout", 0),
        NDRByteField("DoNotWait", 0),
    ]


class RpcWinStationSendMessage_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pResult", 0),
        NDRIntField("pResponse", 0),
        NDRIntField("status", 0),
    ]


class RpcLogonIdFromWinStationName_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRConfStrLenFieldUtf16(
            "pWinStationName", "", size_is=lambda pkt: pkt.NameSize
        ),
        NDRIntField("NameSize", None, size_of="pWinStationName"),
    ]


class RpcLogonIdFromWinStationName_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pResult", 0),
        NDRIntField("pLogonId", 0),
        NDRIntField("status", 0),
    ]


class RpcWinStationNameFromLogonId_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("LoginId", 0),
        NDRConfStrLenFieldUtf16(
            "pWinStationName", "", size_is=lambda pkt: pkt.NameSize
        ),
        NDRIntField("NameSize", None, size_of="pWinStationName"),
    ]


class RpcWinStationNameFromLogonId_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pResult", 0),
        NDRConfStrLenFieldUtf16(
            "pWinStationName", "", size_is=lambda pkt: pkt.NameSize
        ),
        NDRIntField("status", 0),
    ]


class RpcWinStationConnect_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("ClientLogonId", 0),
        NDRIntField("ConnectLogonId", 0),
        NDRIntField("TargetLogonId", 0),
        NDRConfStrLenFieldUtf16("pPassword", "", size_is=lambda pkt: pkt.PasswordSize),
        NDRIntField("PasswordSize", None, size_of="pPassword"),
        NDRByteField("Wait", 0),
    ]


class RpcWinStationConnect_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class RpcWinStationDisconnect_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("LogonId", 0),
        NDRByteField("bWait", 0),
    ]


class RpcWinStationDisconnect_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class RpcWinStationReset_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("LogonId", 0),
        NDRByteField("bWait", 0),
    ]


class RpcWinStationReset_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class RpcWinStationShutdownSystem_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("ClientLogonId", 0),
        NDRIntField("ShutdownFlags", 0),
    ]


class RpcWinStationShutdownSystem_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class RpcWinStationWaitSystemEvent_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("EventMask", 0),
    ]


class RpcWinStationWaitSystemEvent_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pResult", 0),
        NDRIntField("pEventFlags", 0),
        NDRIntField("status", 0),
    ]


class RpcWinStationShadow_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("LogonId", 0),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16(
                "pTargetServerName", "", size_is=lambda pkt: pkt.NameSize
            )
        ),
        NDRIntField("NameSize", None, size_of="pTargetServerName"),
        NDRIntField("TargetLogonId", 0),
        NDRByteField("HotKeyVk", 0),
        NDRShortField("HotkeyModifiers", 0),
    ]


class RpcWinStationShadow_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class RpcWinStationBreakPoint_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("LogonId", 0),
        NDRByteField("KernelFlag", 0),
    ]


class RpcWinStationBreakPoint_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class RpcWinStationReadRegistry_Request(NDRPacket):
    fields_desc = [NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle)]


class RpcWinStationReadRegistry_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class OldRpcWinStationEnumerateProcesses_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("ByteCount", 0),
    ]


class OldRpcWinStationEnumerateProcesses_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pResult", 0),
        NDRConfStrLenField("pProcessBuffer", "", size_is=lambda pkt: pkt.ByteCount),
        NDRIntField("status", 0),
    ]


class RpcWinStationEnumerateProcesses_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("ByteCount", 0),
    ]


class RpcWinStationEnumerateProcesses_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pResult", 0),
        NDRConfStrLenField("pProcessBuffer", "", size_is=lambda pkt: pkt.ByteCount),
        NDRIntField("status", 0),
    ]


class RpcWinStationTerminateProcess_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("ProcessId", 0),
        NDRIntField("ExitCode", 0),
    ]


class RpcWinStationTerminateProcess_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class LARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("QuadPart", 0)]


class TS_UNICODE_STRING(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Length", None, size_of="Buffer"),
        NDRShortField("MaximumLength", None, size_of="Buffer"),
        NDRFullPointerField(
            NDRConfVarStrLenFieldUtf16(
                "Buffer",
                "",
                size_is=lambda pkt: pkt.MaximumLength,
                length_is=lambda pkt: pkt.Length,
            ),
            deferred=True,
        ),
    ]


class PTS_SYS_PROCESS_INFORMATION(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRIntField("NextEntryOffset", 0),
        NDRIntField("NumberOfThreads", 0),
        NDRPacketField("SpareLi1", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("SpareLi2", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("SpareLi3", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("CreateTime", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("UserTime", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("KernelTime", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("ImageName", TS_UNICODE_STRING(), TS_UNICODE_STRING),
        NDRSignedIntField("BasePriority", 0),
        NDRIntField("UniqueProcessId", 0),
        NDRIntField("InheritedFromUniqueProcessId", 0),
        NDRIntField("HandleCount", 0),
        NDRIntField("SessionId", 0),
        NDRIntField("SpareUl3", 0),
        NDRInt3264Field("PeakVirtualSize", 0),
        NDRInt3264Field("VirtualSize", 0),
        NDRIntField("PageFaultCount", 0),
        NDRIntField("PeakWorkingSetSize", 0),
        NDRIntField("WorkingSetSize", 0),
        NDRInt3264Field("QuotaPeakPagedPoolUsage", 0),
        NDRInt3264Field("QuotaPagedPoolUsage", 0),
        NDRInt3264Field("QuotaPeakNonPagedPoolUsage", 0),
        NDRInt3264Field("QuotaNonPagedPoolUsage", 0),
        NDRInt3264Field("PagefileUsage", 0),
        NDRInt3264Field("PeakPagefileUsage", 0),
        NDRInt3264Field("PrivatePageCount", 0),
    ]


class PTS_ALL_PROCESSES_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pTsProcessInfo",
                PTS_SYS_PROCESS_INFORMATION(),
                PTS_SYS_PROCESS_INFORMATION,
            ),
            deferred=True,
        ),
        NDRIntField("SizeOfSid", None, size_of="pSid"),
        NDRFullPointerField(
            NDRConfStrLenField("pSid", "", size_is=lambda pkt: pkt.SizeOfSid),
            deferred=True,
        ),
    ]


class RpcWinStationGetAllProcesses_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Level", 0),
        NDRIntField("pNumberOfProcesses", 0),
    ]


class RpcWinStationGetAllProcesses_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pResult", 0),
        NDRIntField("pNumberOfProcesses", None, size_of="ppTsAllProcessesInfo"),
        NDRConfPacketListField(
            "ppTsAllProcessesInfo",
            [],
            PTS_ALL_PROCESSES_INFO,
            size_is=lambda pkt: pkt.pNumberOfProcesses,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class RpcWinStationGetProcessSid_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwUniqueProcessId", 0),
        NDRPacketField("ProcessStartTime", LARGE_INTEGER(), LARGE_INTEGER),
        NDRFullPointerField(
            NDRConfStrLenField("pProcessUserSid", "", size_is=lambda pkt: pkt.dwSidSize)
        ),
        NDRIntField("dwSidSize", None, size_of="pProcessUserSid"),
        NDRIntField("pdwSizeNeeded", 0),
    ]


class RpcWinStationGetProcessSid_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("pResult", 0),
        NDRFullPointerField(
            NDRConfStrLenField("pProcessUserSid", "", size_is=lambda pkt: pkt.dwSidSize)
        ),
        NDRIntField("pdwSizeNeeded", 0),
        NDRIntField("status", 0),
    ]


class TS_COUNTER_HEADER(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwCounterID", 0), NDRIntField("bResult", 0)]


class PTS_COUNTER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRPacketField("counterHead", TS_COUNTER_HEADER(), TS_COUNTER_HEADER),
        NDRIntField("dwValue", 0),
        NDRPacketField("startTime", LARGE_INTEGER(), LARGE_INTEGER),
    ]


class RpcWinStationGetTermSrvCountersValue_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwEntries", None, size_of="pCounter"),
        NDRConfPacketListField(
            "pCounter", [PTS_COUNTER()], PTS_COUNTER, size_is=lambda pkt: pkt.dwEntries
        ),
    ]


class RpcWinStationGetTermSrvCountersValue_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pResult", 0),
        NDRConfPacketListField(
            "pCounter", [PTS_COUNTER()], PTS_COUNTER, size_is=lambda pkt: pkt.dwEntries
        ),
        NDRIntField("status", 0),
    ]


class RpcWinStationReInitializeSecurity_Request(NDRPacket):
    fields_desc = [NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle)]


class RpcWinStationReInitializeSecurity_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class RpcWinStationGetLanAdapterName_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("PdNameSize", None, size_of="pPdName"),
        NDRConfStrLenFieldUtf16("pPdName", "", size_is=lambda pkt: pkt.PdNameSize),
        NDRIntField("LanAdapter", 0),
    ]


class RpcWinStationGetLanAdapterName_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pResult", 0),
        NDRIntField("pLength", None, size_of="ppLanAdapter"),
        NDRConfStrLenFieldUtf16("ppLanAdapter", "", size_is=lambda pkt: pkt.pLength),
        NDRIntField("status", 0),
    ]


class RpcWinStationUpdateSettings_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("SettingsClass", 0),
        NDRIntField("SettingsParameters", 0),
    ]


class RpcWinStationUpdateSettings_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class RpcWinStationShadowStop_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("LogonId", 0),
        NDRByteField("bWait", 0),
    ]


class RpcWinStationShadowStop_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class RpcWinStationCloseServerEx_Request(NDRPacket):
    fields_desc = [NDRPacketField("phServer", NDRContextHandle(), NDRContextHandle)]


class RpcWinStationCloseServerEx_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("phServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("pResult", 0),
        NDRIntField("status", 0),
    ]


class RpcWinStationIsHelpAssistantSession_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("SessionId", 0),
    ]


class RpcWinStationIsHelpAssistantSession_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class RpcWinStationGetMachinePolicy_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRConfStrLenField("pPolicy", "", size_is=lambda pkt: pkt.bufferSize),
        NDRIntField("bufferSize", None, size_of="pPolicy"),
    ]


class RpcWinStationGetMachinePolicy_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("pPolicy", "", size_is=lambda pkt: pkt.bufferSize),
        NDRIntField("status", 0),
    ]


class RpcWinStationCheckLoopBack_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("ClientLogonId", 0),
        NDRIntField("TargetLogonId", 0),
        NDRConfStrLenFieldUtf16(
            "pTargetServerName", "", size_is=lambda pkt: pkt.NameSize
        ),
        NDRIntField("NameSize", None, size_of="pTargetServerName"),
    ]


class RpcWinStationCheckLoopBack_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class RpcConnectCallback_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("TimeOut", 0),
        NDRIntField("AddressType", 0),
        NDRConfStrLenField("pAddress", "", size_is=lambda pkt: pkt.AddressSize),
        NDRIntField("AddressSize", None, size_of="pAddress"),
    ]


class RpcConnectCallback_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


class NT6_TS_UNICODE_STRING(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Length", None, size_of="Buffer", adjust=lambda _, x: (x * 2)),
        NDRShortField(
            "MaximumLength", None, size_of="Buffer", adjust=lambda _, x: (x * 2)
        ),
        NDRFullPointerField(
            NDRConfVarStrLenFieldUtf16(
                "Buffer",
                "",
                size_is=lambda pkt: (pkt.MaximumLength // 2),
                length_is=lambda pkt: (pkt.Length // 2),
            ),
            deferred=True,
        ),
    ]


class PTS_SYS_PROCESS_INFORMATION_NT6(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRIntField("NextEntryOffset", 0),
        NDRIntField("NumberOfThreads", 0),
        NDRPacketField("SpareLi1", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("SpareLi2", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("SpareLi3", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("CreateTime", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("UserTime", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("KernelTime", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("ImageName", NT6_TS_UNICODE_STRING(), NT6_TS_UNICODE_STRING),
        NDRSignedIntField("BasePriority", 0),
        NDRIntField("UniqueProcessId", 0),
        NDRIntField("InheritedFromUniqueProcessId", 0),
        NDRIntField("HandleCount", 0),
        NDRIntField("SessionId", 0),
        NDRIntField("SpareUl3", 0),
        NDRInt3264Field("PeakVirtualSize", 0),
        NDRInt3264Field("VirtualSize", 0),
        NDRIntField("PageFaultCount", 0),
        NDRIntField("PeakWorkingSetSize", 0),
        NDRIntField("WorkingSetSize", 0),
        NDRInt3264Field("QuotaPeakPagedPoolUsage", 0),
        NDRInt3264Field("QuotaPagedPoolUsage", 0),
        NDRInt3264Field("QuotaPeakNonPagedPoolUsage", 0),
        NDRInt3264Field("QuotaNonPagedPoolUsage", 0),
        NDRInt3264Field("PagefileUsage", 0),
        NDRInt3264Field("PeakPagefileUsage", 0),
        NDRInt3264Field("PrivatePageCount", 0),
    ]


class PTS_ALL_PROCESSES_INFO_NT6(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pTsProcessInfo",
                PTS_SYS_PROCESS_INFORMATION_NT6(),
                PTS_SYS_PROCESS_INFORMATION_NT6,
            ),
            deferred=True,
        ),
        NDRIntField("SizeOfSid", None, size_of="pSid"),
        NDRFullPointerField(
            NDRConfStrLenField("pSid", "", size_is=lambda pkt: pkt.SizeOfSid),
            deferred=True,
        ),
    ]


class RpcWinStationGetAllProcesses_NT6_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Level", 0),
        NDRIntField("pNumberOfProcesses", 0),
    ]


class RpcWinStationGetAllProcesses_NT6_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pResult", 0),
        NDRIntField("pNumberOfProcesses", None, size_of="ppTsAllProcessesInfo"),
        NDRConfPacketListField(
            "ppTsAllProcessesInfo",
            [],
            PTS_ALL_PROCESSES_INFO_NT6,
            size_is=lambda pkt: pkt.pNumberOfProcesses,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class RpcWinStationOpenSessionDirectory_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hServer", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrNullFieldUtf16("pszServerName", ""),
    ]


class RpcWinStationOpenSessionDirectory_Response(NDRPacket):
    fields_desc = [NDRIntField("pResult", 0), NDRIntField("status", 0)]


ICAAPI_OPNUMS = {
    0: DceRpcOp(RpcWinStationOpenServer_Request, RpcWinStationOpenServer_Response),
    1: DceRpcOp(RpcWinStationCloseServer_Request, RpcWinStationCloseServer_Response),
    2: DceRpcOp(RpcIcaServerPing_Request, RpcIcaServerPing_Response),
    3: DceRpcOp(RpcWinStationEnumerate_Request, RpcWinStationEnumerate_Response),
    4: DceRpcOp(RpcWinStationRename_Request, RpcWinStationRename_Response),
    5: DceRpcOp(
        RpcWinStationQueryInformation_Request, RpcWinStationQueryInformation_Response
    ),
    6: DceRpcOp(
        RpcWinStationSetInformation_Request, RpcWinStationSetInformation_Response
    ),
    7: DceRpcOp(RpcWinStationSendMessage_Request, RpcWinStationSendMessage_Response),
    8: DceRpcOp(
        RpcLogonIdFromWinStationName_Request, RpcLogonIdFromWinStationName_Response
    ),
    9: DceRpcOp(
        RpcWinStationNameFromLogonId_Request, RpcWinStationNameFromLogonId_Response
    ),
    10: DceRpcOp(RpcWinStationConnect_Request, RpcWinStationConnect_Response),
    # 11: Opnum11NotUsedOnWire,
    # 12: Opnum12NotUsedOnWire,
    13: DceRpcOp(RpcWinStationDisconnect_Request, RpcWinStationDisconnect_Response),
    14: DceRpcOp(RpcWinStationReset_Request, RpcWinStationReset_Response),
    15: DceRpcOp(
        RpcWinStationShutdownSystem_Request, RpcWinStationShutdownSystem_Response
    ),
    16: DceRpcOp(
        RpcWinStationWaitSystemEvent_Request, RpcWinStationWaitSystemEvent_Response
    ),
    17: DceRpcOp(RpcWinStationShadow_Request, RpcWinStationShadow_Response),
    # 18: Opnum18NotUsedOnWire,
    # 19: Opnum19NotUsedOnWire,
    # 20: Opnum20NotUsedOnWire,
    # 21: Opnum21NotUsedOnWire,
    # 22: Opnum22NotUsedOnWire,
    # 23: Opnum23NotUsedOnWire,
    # 24: Opnum24NotUsedOnWire,
    # 25: Opnum25NotUsedOnWire,
    # 26: Opnum26NotUsedOnWire,
    # 27: Opnum27NotUsedOnWire,
    # 28: Opnum28NotUsedOnWire,
    29: DceRpcOp(RpcWinStationBreakPoint_Request, RpcWinStationBreakPoint_Response),
    30: DceRpcOp(RpcWinStationReadRegistry_Request, RpcWinStationReadRegistry_Response),
    # 31: Opnum31NotUsedOnWire,
    # 32: Opnum32NotUsedOnWire,
    # 33: Opnum33NotUsedOnWire,
    34: DceRpcOp(
        OldRpcWinStationEnumerateProcesses_Request,
        OldRpcWinStationEnumerateProcesses_Response,
    ),
    # 35: Opnum35NotUsedOnWire,
    36: DceRpcOp(
        RpcWinStationEnumerateProcesses_Request,
        RpcWinStationEnumerateProcesses_Response,
    ),
    37: DceRpcOp(
        RpcWinStationTerminateProcess_Request, RpcWinStationTerminateProcess_Response
    ),
    # 38: Opnum38NotUsedOnWire,
    # 39: Opnum39NotUsedOnWire,
    # 40: Opnum40NotUsedOnWire,
    # 41: Opnum41NotUsedOnWire,
    # 42: Opnum42NotUsedOnWire,
    43: DceRpcOp(
        RpcWinStationGetAllProcesses_Request, RpcWinStationGetAllProcesses_Response
    ),
    44: DceRpcOp(
        RpcWinStationGetProcessSid_Request, RpcWinStationGetProcessSid_Response
    ),
    45: DceRpcOp(
        RpcWinStationGetTermSrvCountersValue_Request,
        RpcWinStationGetTermSrvCountersValue_Response,
    ),
    46: DceRpcOp(
        RpcWinStationReInitializeSecurity_Request,
        RpcWinStationReInitializeSecurity_Response,
    ),
    # 47: Opnum47NotUsedOnWire,
    # 48: Opnum48NotUsedOnWire,
    # 49: Opnum49NotUsedOnWire,
    # 50: Opnum50NotUsedOnWire,
    # 51: Opnum51NotUsedOnWire,
    # 52: Opnum52NotUsedOnWire,
    53: DceRpcOp(
        RpcWinStationGetLanAdapterName_Request, RpcWinStationGetLanAdapterName_Response
    ),
    # 54: Opnum54NotUsedOnWire,
    # 55: Opnum55NotUsedOnWire,
    # 56: Opnum56NotUsedOnWire,
    # 57: Opnum57NotUsedOnWire,
    58: DceRpcOp(
        RpcWinStationUpdateSettings_Request, RpcWinStationUpdateSettings_Response
    ),
    59: DceRpcOp(RpcWinStationShadowStop_Request, RpcWinStationShadowStop_Response),
    60: DceRpcOp(
        RpcWinStationCloseServerEx_Request, RpcWinStationCloseServerEx_Response
    ),
    61: DceRpcOp(
        RpcWinStationIsHelpAssistantSession_Request,
        RpcWinStationIsHelpAssistantSession_Response,
    ),
    62: DceRpcOp(
        RpcWinStationGetMachinePolicy_Request, RpcWinStationGetMachinePolicy_Response
    ),
    # 63: Opnum63NotUsedOnWire,
    # 64: Opnum64NotUsedOnWire,
    65: DceRpcOp(
        RpcWinStationCheckLoopBack_Request, RpcWinStationCheckLoopBack_Response
    ),
    66: DceRpcOp(RpcConnectCallback_Request, RpcConnectCallback_Response),
    # 67: Opnum67NotUsedOnWire,
    # 68: Opnum68NotUsedOnWire,
    # 69: Opnum69NotUsedOnWire,
    70: DceRpcOp(
        RpcWinStationGetAllProcesses_NT6_Request,
        RpcWinStationGetAllProcesses_NT6_Response,
    ),
    # 71: Opnum71NotUsedOnWire,
    # 72: Opnum72NotUsedOnWire,
    # 73: Opnum73NotUsedOnWire,
    # 74: Opnum74NotUsedOnWire,
    75: DceRpcOp(
        RpcWinStationOpenSessionDirectory_Request,
        RpcWinStationOpenSessionDirectory_Response,
    ),
}
register_dcerpc_interface(
    name="IcaApi",
    uuid=uuid.UUID("5ca4a760-ebb1-11cf-8611-00a0245420ed"),
    version="1.0",
    opnums=ICAAPI_OPNUMS,
)



### ms-icpr.idl
"""
RPC definitions for the following interfaces:
- ICertPassage (v0.0): 91ae6020-9e3c-11cf-8d7c-00aa00c091be
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfStrLenField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    register_dcerpc_interface,
)


class CERTTRANSBLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cb", None, size_of="pb"),
        NDRFullPointerField(
            NDRConfStrLenField("pb", "", size_is=lambda pkt: pkt.cb), deferred=True
        ),
    ]


class CertServerRequest_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwFlags", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("pdwRequestId", 0),
        NDRPacketField("pctbAttribs", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRPacketField("pctbRequest", CERTTRANSBLOB(), CERTTRANSBLOB),
    ]


class CertServerRequest_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pdwRequestId", 0),
        NDRIntField("pdwDisposition", 0),
        NDRPacketField("pctbCert", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRPacketField("pctbEncodedCert", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRPacketField("pctbDispositionMessage", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRIntField("status", 0),
    ]


ICERTPASSAGE_OPNUMS = {
    0: DceRpcOp(CertServerRequest_Request, CertServerRequest_Response)
}
register_dcerpc_interface(
    name="ICertPassage",
    uuid=uuid.UUID("91ae6020-9e3c-11cf-8d7c-00aa00c091be"),
    version="0.0",
    opnums=ICERTPASSAGE_OPNUMS,
)



### ms-bpau.idl
"""
RPC definitions for the following interfaces:
- BitsPeerAuth (v1.0): e3d0d746-d2af-40fd-8a7a-0d7078bb7092
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfStrLenField,
    NDRIntField,
    register_dcerpc_interface,
)


class ExchangePublicKeys_Request(NDRPacket):
    fields_desc = [
        NDRIntField("ClientKeyLength", None, size_of="ClientKey"),
        NDRConfStrLenField("ClientKey", "", size_is=lambda pkt: pkt.ClientKeyLength),
    ]


class ExchangePublicKeys_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pServerKeyLength", None, size_of="pServerKey"),
        NDRConfStrLenField("pServerKey", "", size_is=lambda pkt: pkt.pServerKeyLength),
        NDRIntField("status", 0),
    ]


BITSPEERAUTH_OPNUMS = {
    0: DceRpcOp(ExchangePublicKeys_Request, ExchangePublicKeys_Response)
}
register_dcerpc_interface(
    name="BitsPeerAuth",
    uuid=uuid.UUID("e3d0d746-d2af-40fd-8a7a-0d7078bb7092"),
    version="1.0",
    opnums=BITSPEERAUTH_OPNUMS,
)



### mc-mqac.idl
"""
RPC definitions for the following interfaces:
- ITransaction (v0.0): 0fb15084-af41-11ce-bd2b-204c4f4f5020
- IEnumConnections (v0.0): B196B287-BAB4-101A-B69C-00AA00341D07
- IConnectionPointContainer (v0.0): B196B284-BAB4-101A-B69C-00AA00341D07
- IConnectionPoint (v0.0): B196B286-BAB4-101A-B69C-00AA00341D07
- IEnumConnectionPoints (v0.0): B196B285-BAB4-101A-B69C-00AA00341D07
- IMSMQQuery (v0.0): None
- IMSMQQueueInfo (v0.0): None
- IMSMQQueueInfo2 (v0.0): None
- IMSMQQueueInfo3 (v0.0): None
- IMSMQQueueInfo4 (v0.0): None
- IMSMQQueue (v0.0): None
- IMSMQQueue2 (v0.0): None
- IMSMQQueue3 (v0.0): None
- IMSMQQueue4 (v0.0): None
- IMSMQMessage (v0.0): None
- IMSMQQueueInfos (v0.0): None
- IMSMQQueueInfos2 (v0.0): None
- IMSMQQueueInfos3 (v0.0): None
- IMSMQQueueInfos4 (v0.0): None
- IMSMQEvent (v0.0): None
- IMSMQEvent2 (v0.0): None
- IMSMQEvent3 (v0.0): None
- IMSMQTransaction (v0.0): None
- IMSMQCoordinatedTransactionDispenser (v0.0): None
- IMSMQTransactionDispenser (v0.0): None
- IMSMQQuery2 (v0.0): None
- IMSMQQuery3 (v0.0): None
- IMSMQQuery4 (v0.0): None
- IMSMQMessage2 (v0.0): None
- IMSMQMessage3 (v0.0): None
- IMSMQMessage4 (v0.0): None
- IMSMQPrivateEvent (v0.0): None
- IMSMQTransaction2 (v0.0): None
- IMSMQTransaction3 (v0.0): None
- IMSMQCoordinatedTransactionDispenser2 (v0.0): None
- IMSMQCoordinatedTransactionDispenser3 (v0.0): None
- IMSMQTransactionDispenser2 (v0.0): None
- IMSMQTransactionDispenser3 (v0.0): None
- IMSMQApplication (v0.0): None
- IMSMQApplication2 (v0.0): None
- IMSMQApplication3 (v0.0): None
- IMSMQDestination (v0.0): None
- IMSMQPrivateDestination (v0.0): None
- IMSMQCollection (v0.0): None
- IMSMQManagement (v0.0): None
- IMSMQOutgoingQueueManagement (v0.0): None
- IMSMQQueueManagement (v0.0): None
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import PacketListField, StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfStrLenField,
    NDRConfVarPacketListField,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedShortField,
    register_com_interface,
)


class Commit_Request(NDRPacket):
    fields_desc = [
        NDRSignedShortField("fRetaining", 0),
        NDRIntField("grfTC", 0),
        NDRIntField("grfRM", 0),
    ]


class Commit_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class BOID(NDRPacket):
    fields_desc = [StrFixedLenField("rgb", "", length=16)]


class Abort_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("pboidReason", BOID(), BOID)),
        NDRSignedShortField("fRetaining", 0),
        NDRSignedShortField("fAsync", 0),
    ]


class Abort_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class XACTTRANSINFO(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("uow", BOID(), BOID),
        NDRSignedIntField("isoLevel", 0),
        NDRIntField("isoFlags", 0),
        NDRIntField("grfTCSupported", 0),
        NDRIntField("grfRMSupported", 0),
        NDRIntField("grfTCSupportedRetaining", 0),
        NDRIntField("grfRMSupportedRetaining", 0),
    ]


class GetTransactionInfo_Request(NDRPacket):
    fields_desc = []


class GetTransactionInfo_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pinfo", XACTTRANSINFO(), XACTTRANSINFO),
        NDRIntField("status", 0),
    ]


ITRANSACTION_OPNUMS = {
    0: DceRpcOp(Commit_Request, Commit_Response),
    1: DceRpcOp(Abort_Request, Abort_Response),
    2: DceRpcOp(GetTransactionInfo_Request, GetTransactionInfo_Response),
}
register_com_interface(
    name="ITransaction",
    uuid=uuid.UUID("0fb15084-af41-11ce-bd2b-204c4f4f5020"),
    opnums=ITRANSACTION_OPNUMS,
)


class MInterfacePointer(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["abData"]
    fields_desc = [
        NDRIntField("ulCntData", None, size_of="abData"),
        NDRConfStrLenField(
            "abData", "", size_is=lambda pkt: pkt.ulCntData, conformant_in_struct=True
        ),
    ]


class CONNECTDATA(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pUnk", MInterfacePointer(), MInterfacePointer),
            deferred=True,
        ),
        NDRIntField("dwCookie", 0),
    ]


class Next_Request(NDRPacket):
    fields_desc = [NDRIntField("cConnections", 0)]


class Next_Response(NDRPacket):
    fields_desc = [
        NDRConfVarPacketListField(
            "rgcd",
            [CONNECTDATA()],
            CONNECTDATA,
            size_is=lambda pkt: pkt.cConnections,
            length_is=lambda pkt: pkt.pcFetched,
        ),
        NDRIntField("pcFetched", None, size_of="rgcd"),
        NDRIntField("status", 0),
    ]


class Skip_Request(NDRPacket):
    fields_desc = [NDRIntField("cConnections", 0)]


class Skip_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Reset_Request(NDRPacket):
    fields_desc = []


class Reset_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Clone_Request(NDRPacket):
    fields_desc = []


class Clone_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppEnum", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


IENUMCONNECTIONS_OPNUMS = {
    0: DceRpcOp(Next_Request, Next_Response),
    1: DceRpcOp(Skip_Request, Skip_Response),
    2: DceRpcOp(Reset_Request, Reset_Response),
    3: DceRpcOp(Clone_Request, Clone_Response),
}
register_com_interface(
    name="IEnumConnections",
    uuid=uuid.UUID("B196B287-BAB4-101A-B69C-00AA00341D07"),
    opnums=IENUMCONNECTIONS_OPNUMS,
)


class EnumConnectionPoints_Request(NDRPacket):
    fields_desc = []


class EnumConnectionPoints_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppEnum", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class FindConnectionPoint_Request(NDRPacket):
    fields_desc = [NDRPacketField("riid", GUID(), GUID)]


class FindConnectionPoint_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppCP", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


ICONNECTIONPOINTCONTAINER_OPNUMS = {
    0: DceRpcOp(EnumConnectionPoints_Request, EnumConnectionPoints_Response),
    1: DceRpcOp(FindConnectionPoint_Request, FindConnectionPoint_Response),
}
register_com_interface(
    name="IConnectionPointContainer",
    uuid=uuid.UUID("B196B284-BAB4-101A-B69C-00AA00341D07"),
    opnums=ICONNECTIONPOINTCONTAINER_OPNUMS,
)


class GetConnectionInterface_Request(NDRPacket):
    fields_desc = []


class GetConnectionInterface_Response(NDRPacket):
    fields_desc = [NDRPacketField("pIID", GUID(), GUID), NDRIntField("status", 0)]


class GetConnectionPointContainer_Request(NDRPacket):
    fields_desc = []


class GetConnectionPointContainer_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppCPC", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class Advise_Request(NDRPacket):
    fields_desc = [NDRPacketField("pUnkSink", MInterfacePointer(), MInterfacePointer)]


class Advise_Response(NDRPacket):
    fields_desc = [NDRIntField("pdwCookie", 0), NDRIntField("status", 0)]


class Unadvise_Request(NDRPacket):
    fields_desc = [NDRIntField("dwCookie", 0)]


class Unadvise_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EnumConnections_Request(NDRPacket):
    fields_desc = []


class EnumConnections_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppEnum", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


ICONNECTIONPOINT_OPNUMS = {
    0: DceRpcOp(GetConnectionInterface_Request, GetConnectionInterface_Response),
    1: DceRpcOp(
        GetConnectionPointContainer_Request, GetConnectionPointContainer_Response
    ),
    2: DceRpcOp(Advise_Request, Advise_Response),
    3: DceRpcOp(Unadvise_Request, Unadvise_Response),
    4: DceRpcOp(EnumConnections_Request, EnumConnections_Response),
}
register_com_interface(
    name="IConnectionPoint",
    uuid=uuid.UUID("B196B286-BAB4-101A-B69C-00AA00341D07"),
    opnums=ICONNECTIONPOINT_OPNUMS,
)


class Next_Request(NDRPacket):
    fields_desc = [NDRIntField("cConnections", 0)]


class Next_Response(NDRPacket):
    fields_desc = [
        NDRConfVarPacketListField(
            "ppCP",
            [],
            MInterfacePointer,
            size_is=lambda pkt: pkt.cConnections,
            length_is=lambda pkt: pkt.pcFetched,
            ptr_pack=True,
        ),
        NDRIntField("pcFetched", None, size_of="ppCP"),
        NDRIntField("status", 0),
    ]


class Skip_Request(NDRPacket):
    fields_desc = [NDRIntField("cConnections", 0)]


class Skip_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Reset_Request(NDRPacket):
    fields_desc = []


class Reset_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Clone_Request(NDRPacket):
    fields_desc = []


class Clone_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppEnum", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


IENUMCONNECTIONPOINTS_OPNUMS = {
    0: DceRpcOp(Next_Request, Next_Response),
    1: DceRpcOp(Skip_Request, Skip_Response),
    2: DceRpcOp(Reset_Request, Reset_Response),
    3: DceRpcOp(Clone_Request, Clone_Response),
}
register_com_interface(
    name="IEnumConnectionPoints",
    uuid=uuid.UUID("B196B285-BAB4-101A-B69C-00AA00341D07"),
    opnums=IENUMCONNECTIONPOINTS_OPNUMS,
)



### ms-wmi.idl
"""
RPC definitions for the following interfaces:
- IWbemClassObject (v0.0): dc12a681-737f-11cf-884d-00aa004b2e24
- IWbemServices (v0.0): 9556dc99-828c-11cf-a37e-00aa003240c7
- IWbemObjectSink (v0.0): 7c857801-7381-11cf-884d-00aa004b2e24
- IEnumWbemClassObject (v0.0): 027947e1-d731-11ce-a357-000000000001
- IWbemCallResult (v0.0): 44aca675-e8fc-11d0-a07c-00c04fb68820
- IWbemContext (v0.0): 44aca674-e8fc-11d0-a07c-00c04fb68820
- IWbemBackupRestore (v0.0): C49E32C7-BC8B-11d2-85D4-00105A1F8304
- IWbemBackupRestoreEx (v0.0): A359DEC5-E813-4834-8A2A-BA7F1D777D76
- IWbemLoginClientID (v0.0): d4781cd6-e5d3-44df-ad94-930efe48a887
- IWbemLevel1Login (v0.0): F309AD18-D86A-11d0-A075-00C04FB68820
- IWbemLoginHelper (v0.0): 541679AB-2E5F-11d3-B34E-00104BCC4B4A
- IWbemQualifierSet (v0.0): None
- IWbemRemoteRefresher (v0.0): F1E9C5B2-F59B-11d2-B362-00105A1F8177
- IWbemRefreshingServices (v0.0): 2C9273E0-1DC3-11d3-B364-00105A1F8177
- IWbemWCOSmartEnum (v0.0): 423EC01E-2E35-11d2-B604-00104B703EFD
- IWbemFetchSmartEnum (v0.0): 1C1C45EE-4395-11d2-B60B-00104B703EFD
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import PacketListField, StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRConfVarPacketListField,
    NDRConfVarStrLenField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRIntEnumField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    NDRSignedByteField,
    NDRSignedIntField,
    NDRUnionField,
    register_com_interface,
    register_dcerpc_interface,
)

IWBEMCLASSOBJECT_OPNUMS = {}
register_com_interface(
    name="IWbemClassObject",
    uuid=uuid.UUID("dc12a681-737f-11cf-884d-00aa004b2e24"),
    opnums=IWBEMCLASSOBJECT_OPNUMS,
)


class FLAGGED_WORD_BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["asData"]
    fields_desc = [
        NDRIntField("cBytes", 0),
        NDRIntField("clSize", None, size_of="asData"),
        NDRConfStrLenFieldUtf16(
            "asData", "", size_is=lambda pkt: pkt.clSize, conformant_in_struct=True
        ),
    ]


class MInterfacePointer(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["abData"]
    fields_desc = [
        NDRIntField("ulCntData", None, size_of="abData"),
        NDRConfStrLenField(
            "abData", "", size_is=lambda pkt: pkt.ulCntData, conformant_in_struct=True
        ),
    ]


class OpenNamespace_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strNamespace", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField(
                    "ppWorkingNamespace", MInterfacePointer(), MInterfacePointer
                )
            )
        ),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppResult", MInterfacePointer(), MInterfacePointer)
            )
        ),
    ]


class OpenNamespace_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField(
                    "ppWorkingNamespace", MInterfacePointer(), MInterfacePointer
                )
            )
        ),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppResult", MInterfacePointer(), MInterfacePointer)
            )
        ),
        NDRIntField("status", 0),
    ]


class CancelAsyncCall_Request(NDRPacket):
    fields_desc = [NDRPacketField("pSink", MInterfacePointer(), MInterfacePointer)]


class CancelAsyncCall_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class QueryObjectSink_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("lFlags", 0)]


class QueryObjectSink_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppResponseHandler", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class GetObject_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strObjectPath", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppObject", MInterfacePointer(), MInterfacePointer)
            )
        ),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppCallResult", MInterfacePointer(), MInterfacePointer)
            )
        ),
    ]


class GetObject_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppObject", MInterfacePointer(), MInterfacePointer)
            )
        ),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppCallResult", MInterfacePointer(), MInterfacePointer)
            )
        ),
        NDRIntField("status", 0),
    ]


class GetObjectAsync_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strObjectPath", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRPacketField("pResponseHandler", MInterfacePointer(), MInterfacePointer),
    ]


class GetObjectAsync_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class PutClass_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pObject", MInterfacePointer(), MInterfacePointer),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppCallResult", MInterfacePointer(), MInterfacePointer)
            )
        ),
    ]


class PutClass_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppCallResult", MInterfacePointer(), MInterfacePointer)
            )
        ),
        NDRIntField("status", 0),
    ]


class PutClassAsync_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pObject", MInterfacePointer(), MInterfacePointer),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRPacketField("pResponseHandler", MInterfacePointer(), MInterfacePointer),
    ]


class PutClassAsync_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteClass_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strClass", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppCallResult", MInterfacePointer(), MInterfacePointer)
            )
        ),
    ]


class DeleteClass_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppCallResult", MInterfacePointer(), MInterfacePointer)
            )
        ),
        NDRIntField("status", 0),
    ]


class DeleteClassAsync_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strClass", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRPacketField("pResponseHandler", MInterfacePointer(), MInterfacePointer),
    ]


class DeleteClassAsync_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CreateClassEnum_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strSuperclass", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
    ]


class CreateClassEnum_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppEnum", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class CreateClassEnumAsync_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strSuperclass", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRPacketField("pResponseHandler", MInterfacePointer(), MInterfacePointer),
    ]


class CreateClassEnumAsync_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class PutInstance_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pInst", MInterfacePointer(), MInterfacePointer),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppCallResult", MInterfacePointer(), MInterfacePointer)
            )
        ),
    ]


class PutInstance_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppCallResult", MInterfacePointer(), MInterfacePointer)
            )
        ),
        NDRIntField("status", 0),
    ]


class PutInstanceAsync_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pInst", MInterfacePointer(), MInterfacePointer),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRPacketField("pResponseHandler", MInterfacePointer(), MInterfacePointer),
    ]


class PutInstanceAsync_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteInstance_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strObjectPath", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppCallResult", MInterfacePointer(), MInterfacePointer)
            )
        ),
    ]


class DeleteInstance_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppCallResult", MInterfacePointer(), MInterfacePointer)
            )
        ),
        NDRIntField("status", 0),
    ]


class DeleteInstanceAsync_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strObjectPath", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRPacketField("pResponseHandler", MInterfacePointer(), MInterfacePointer),
    ]


class DeleteInstanceAsync_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CreateInstanceEnum_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strSuperClass", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
    ]


class CreateInstanceEnum_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppEnum", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class CreateInstanceEnumAsync_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strSuperClass", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRPacketField("pResponseHandler", MInterfacePointer(), MInterfacePointer),
    ]


class CreateInstanceEnumAsync_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ExecQuery_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strQueryLanguage", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("strQuery", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
    ]


class ExecQuery_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppEnum", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class ExecQueryAsync_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strQueryLanguage", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("strQuery", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRPacketField("pResponseHandler", MInterfacePointer(), MInterfacePointer),
    ]


class ExecQueryAsync_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ExecNotificationQuery_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strQueryLanguage", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("strQuery", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
    ]


class ExecNotificationQuery_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppEnum", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class ExecNotificationQueryAsync_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strQueryLanguage", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("strQuery", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRPacketField("pResponseHandler", MInterfacePointer(), MInterfacePointer),
    ]


class ExecNotificationQueryAsync_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ExecMethod_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strObjectPath", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("strMethodName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRPacketField("pInParams", MInterfacePointer(), MInterfacePointer),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppOutParams", MInterfacePointer(), MInterfacePointer)
            )
        ),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppCallResult", MInterfacePointer(), MInterfacePointer)
            )
        ),
    ]


class ExecMethod_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppOutParams", MInterfacePointer(), MInterfacePointer)
            )
        ),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppCallResult", MInterfacePointer(), MInterfacePointer)
            )
        ),
        NDRIntField("status", 0),
    ]


class ExecMethodAsync_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("strObjectPath", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("strMethodName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
        NDRPacketField("pInParams", MInterfacePointer(), MInterfacePointer),
        NDRPacketField("pResponseHandler", MInterfacePointer(), MInterfacePointer),
    ]


class ExecMethodAsync_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IWBEMSERVICES_OPNUMS = {
    0: DceRpcOp(OpenNamespace_Request, OpenNamespace_Response),
    1: DceRpcOp(CancelAsyncCall_Request, CancelAsyncCall_Response),
    2: DceRpcOp(QueryObjectSink_Request, QueryObjectSink_Response),
    3: DceRpcOp(GetObject_Request, GetObject_Response),
    4: DceRpcOp(GetObjectAsync_Request, GetObjectAsync_Response),
    5: DceRpcOp(PutClass_Request, PutClass_Response),
    6: DceRpcOp(PutClassAsync_Request, PutClassAsync_Response),
    7: DceRpcOp(DeleteClass_Request, DeleteClass_Response),
    8: DceRpcOp(DeleteClassAsync_Request, DeleteClassAsync_Response),
    9: DceRpcOp(CreateClassEnum_Request, CreateClassEnum_Response),
    10: DceRpcOp(CreateClassEnumAsync_Request, CreateClassEnumAsync_Response),
    11: DceRpcOp(PutInstance_Request, PutInstance_Response),
    12: DceRpcOp(PutInstanceAsync_Request, PutInstanceAsync_Response),
    13: DceRpcOp(DeleteInstance_Request, DeleteInstance_Response),
    14: DceRpcOp(DeleteInstanceAsync_Request, DeleteInstanceAsync_Response),
    15: DceRpcOp(CreateInstanceEnum_Request, CreateInstanceEnum_Response),
    16: DceRpcOp(CreateInstanceEnumAsync_Request, CreateInstanceEnumAsync_Response),
    17: DceRpcOp(ExecQuery_Request, ExecQuery_Response),
    18: DceRpcOp(ExecQueryAsync_Request, ExecQueryAsync_Response),
    19: DceRpcOp(ExecNotificationQuery_Request, ExecNotificationQuery_Response),
    20: DceRpcOp(
        ExecNotificationQueryAsync_Request, ExecNotificationQueryAsync_Response
    ),
    21: DceRpcOp(ExecMethod_Request, ExecMethod_Response),
    22: DceRpcOp(ExecMethodAsync_Request, ExecMethodAsync_Response),
}
register_com_interface(
    name="IWbemServices",
    uuid=uuid.UUID("9556dc99-828c-11cf-a37e-00aa003240c7"),
    opnums=IWBEMSERVICES_OPNUMS,
)


class Indicate_Request(NDRPacket):
    fields_desc = [
        NDRSignedIntField("lObjectCount", None, size_of="apObjArray"),
        NDRConfPacketListField(
            "apObjArray",
            [],
            MInterfacePointer,
            size_is=lambda pkt: pkt.lObjectCount,
            ptr_pack=True,
        ),
    ]


class Indicate_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SetStatus_Request(NDRPacket):
    fields_desc = [
        NDRSignedIntField("lFlags", 0),
        NDRSignedIntField("hResult", 0),
        NDRPacketField("strParam", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("pObjParam", MInterfacePointer(), MInterfacePointer),
    ]


class SetStatus_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IWBEMOBJECTSINK_OPNUMS = {
    0: DceRpcOp(Indicate_Request, Indicate_Response),
    1: DceRpcOp(SetStatus_Request, SetStatus_Response),
}
register_com_interface(
    name="IWbemObjectSink",
    uuid=uuid.UUID("7c857801-7381-11cf-884d-00aa004b2e24"),
    opnums=IWBEMOBJECTSINK_OPNUMS,
)


class Reset_Request(NDRPacket):
    fields_desc = []


class Reset_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Next_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("lTimeout", 0), NDRIntField("uCount", 0)]


class Next_Response(NDRPacket):
    fields_desc = [
        NDRConfVarPacketListField(
            "apObjects",
            [],
            MInterfacePointer,
            size_is=lambda pkt: pkt.uCount,
            length_is=lambda pkt: pkt.puReturned,
            ptr_pack=True,
        ),
        NDRIntField("puReturned", None, size_of="apObjects"),
        NDRIntField("status", 0),
    ]


class NextAsync_Request(NDRPacket):
    fields_desc = [
        NDRIntField("uCount", 0),
        NDRPacketField("pSink", MInterfacePointer(), MInterfacePointer),
    ]


class NextAsync_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Clone_Request(NDRPacket):
    fields_desc = []


class Clone_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppEnum", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class Skip_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("lTimeout", 0), NDRIntField("nCount", 0)]


class Skip_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IENUMWBEMCLASSOBJECT_OPNUMS = {
    0: DceRpcOp(Reset_Request, Reset_Response),
    1: DceRpcOp(Next_Request, Next_Response),
    2: DceRpcOp(NextAsync_Request, NextAsync_Response),
    3: DceRpcOp(Clone_Request, Clone_Response),
    4: DceRpcOp(Skip_Request, Skip_Response),
}
register_com_interface(
    name="IEnumWbemClassObject",
    uuid=uuid.UUID("027947e1-d731-11ce-a357-000000000001"),
    opnums=IENUMWBEMCLASSOBJECT_OPNUMS,
)


class GetResultObject_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("lTimeout", 0)]


class GetResultObject_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppResultObject", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class GetResultString_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("lTimeout", 0)]


class GetResultString_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pstrResultString", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class GetResultServices_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("lTimeout", 0)]


class GetResultServices_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppServices", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class GetCallStatus_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("lTimeout", 0)]


class GetCallStatus_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("plStatus", 0), NDRIntField("status", 0)]


IWBEMCALLRESULT_OPNUMS = {
    0: DceRpcOp(GetResultObject_Request, GetResultObject_Response),
    1: DceRpcOp(GetResultString_Request, GetResultString_Response),
    2: DceRpcOp(GetResultServices_Request, GetResultServices_Response),
    3: DceRpcOp(GetCallStatus_Request, GetCallStatus_Response),
}
register_com_interface(
    name="IWbemCallResult",
    uuid=uuid.UUID("44aca675-e8fc-11d0-a07c-00c04fb68820"),
    opnums=IWBEMCALLRESULT_OPNUMS,
)
IWBEMCONTEXT_OPNUMS = {}
register_com_interface(
    name="IWbemContext",
    uuid=uuid.UUID("44aca674-e8fc-11d0-a07c-00c04fb68820"),
    opnums=IWBEMCONTEXT_OPNUMS,
)


class Backup_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("strBackupToFile", ""),
        NDRSignedIntField("lFlags", 0),
    ]


class Backup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Restore_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("strRestoreFromFile", ""),
        NDRSignedIntField("lFlags", 0),
    ]


class Restore_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IWBEMBACKUPRESTORE_OPNUMS = {
    0: DceRpcOp(Backup_Request, Backup_Response),
    1: DceRpcOp(Restore_Request, Restore_Response),
}
register_com_interface(
    name="IWbemBackupRestore",
    uuid=uuid.UUID("C49E32C7-BC8B-11d2-85D4-00105A1F8304"),
    opnums=IWBEMBACKUPRESTORE_OPNUMS,
)


class Pause_Request(NDRPacket):
    fields_desc = []


class Pause_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Resume_Request(NDRPacket):
    fields_desc = []


class Resume_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IWBEMBACKUPRESTOREEX_OPNUMS = {
    0: DceRpcOp(Pause_Request, Pause_Response),
    1: DceRpcOp(Resume_Request, Resume_Response),
}
register_com_interface(
    name="IWbemBackupRestoreEx",
    uuid=uuid.UUID("A359DEC5-E813-4834-8A2A-BA7F1D777D76"),
    opnums=IWBEMBACKUPRESTOREEX_OPNUMS,
)


class SetClientInfo_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("wszClientMachine", "")),
        NDRSignedIntField("lClientProcId", 0),
        NDRSignedIntField("lReserved", 0),
    ]


class SetClientInfo_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IWBEMLOGINCLIENTID_OPNUMS = {0: DceRpcOp(SetClientInfo_Request, SetClientInfo_Response)}
register_com_interface(
    name="IWbemLoginClientID",
    uuid=uuid.UUID("d4781cd6-e5d3-44df-ad94-930efe48a887"),
    opnums=IWBEMLOGINCLIENTID_OPNUMS,
)


class EstablishPosition_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("reserved1", "")),
        NDRIntField("reserved2", 0),
    ]


class EstablishPosition_Response(NDRPacket):
    fields_desc = [NDRIntField("LocaleVersion", 0), NDRIntField("status", 0)]


class RequestChallenge_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("reserved1", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("reserved2", "")),
    ]


class RequestChallenge_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "reserved3", "", size_is=lambda pkt: 16, length_is=lambda pkt: 16
        ),
        NDRIntField("status", 0),
    ]


class WBEMLogin_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("reserved1", "")),
        NDRConfVarStrLenField(
            "reserved2", "", size_is=lambda pkt: 16, length_is=lambda pkt: 16
        ),
        NDRSignedIntField("reserved3", 0),
        NDRPacketField("reserved4", MInterfacePointer(), MInterfacePointer),
    ]


class WBEMLogin_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("reserved5", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class NTLMLogin_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("wszNetworkResource", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("wszPreferredLocale", "")),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pCtx", MInterfacePointer(), MInterfacePointer),
    ]


class NTLMLogin_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppNamespace", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


IWBEMLEVEL1LOGIN_OPNUMS = {
    0: DceRpcOp(EstablishPosition_Request, EstablishPosition_Response),
    1: DceRpcOp(RequestChallenge_Request, RequestChallenge_Response),
    2: DceRpcOp(WBEMLogin_Request, WBEMLogin_Response),
    3: DceRpcOp(NTLMLogin_Request, NTLMLogin_Response),
}
register_com_interface(
    name="IWbemLevel1Login",
    uuid=uuid.UUID("F309AD18-D86A-11d0-A075-00C04FB68820"),
    opnums=IWBEMLEVEL1LOGIN_OPNUMS,
)


class SetEvent_Request(NDRPacket):
    fields_desc = [NDRSignedByteField("sEventToSet", 0)]


class SetEvent_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IWBEMLOGINHELPER_OPNUMS = {0: DceRpcOp(SetEvent_Request, SetEvent_Response)}
register_com_interface(
    name="IWbemLoginHelper",
    uuid=uuid.UUID("541679AB-2E5F-11d3-B34E-00104BCC4B4A"),
    opnums=IWBEMLOGINHELPER_OPNUMS,
)


class WBEM_INSTANCE_BLOB_TYPE(IntEnum):
    WBEM_BLOB_TYPE_ALL = 2
    WBEM_BLOB_TYPE_ERROR = 3
    WBEM_BLOB_TYPE_ENUM = 4


class WBEM_REFRESHED_OBJECT(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRSignedIntField("m_lRequestId", 0),
        NDRIntEnumField("m_lBlobType", 0, WBEM_INSTANCE_BLOB_TYPE),
        NDRSignedIntField("m_lBlobLength", None, size_of="m_pbBlob"),
        NDRFullPointerField(
            NDRConfStrLenField("m_pbBlob", "", size_is=lambda pkt: pkt.m_lBlobLength),
            deferred=True,
        ),
    ]


class RemoteRefresh_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("lFlags", 0)]


class RemoteRefresh_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("plNumObjects", None, size_of="paObjects"),
        NDRConfPacketListField(
            "paObjects",
            [],
            WBEM_REFRESHED_OBJECT,
            size_is=lambda pkt: pkt.plNumObjects,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class StopRefreshing_Request(NDRPacket):
    fields_desc = [
        NDRSignedIntField("lNumIds", None, size_of="aplIds"),
        NDRConfFieldListField(
            "aplIds", [], NDRSignedIntField("", 0), size_is=lambda pkt: pkt.lNumIds
        ),
        NDRSignedIntField("lFlags", 0),
    ]


class StopRefreshing_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class Opnum5NotUsedOnWire_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("lFlags", 0)]


class Opnum5NotUsedOnWire_Response(NDRPacket):
    fields_desc = [NDRPacketField("pGuid", GUID(), GUID), NDRIntField("status", 0)]


IWBEMREMOTEREFRESHER_OPNUMS = {
    0: DceRpcOp(RemoteRefresh_Request, RemoteRefresh_Response),
    1: DceRpcOp(StopRefreshing_Request, StopRefreshing_Response),
    2: DceRpcOp(Opnum5NotUsedOnWire_Request, Opnum5NotUsedOnWire_Response),
}
register_dcerpc_interface(
    name="IWbemRemoteRefresher",
    uuid=uuid.UUID("F1E9C5B2-F59B-11d2-B362-00105A1F8177"),
    version="0.0",
    opnums=IWBEMREMOTEREFRESHER_OPNUMS,
)


class _WBEM_REFRESHER_ID(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRConfVarStrNullField("m_szMachineName", ""), deferred=True
        ),
        NDRIntField("m_dwProcessId", 0),
        NDRPacketField("m_guidRefresherId", GUID(), GUID),
    ]


class WBEM_REFRESH_TYPE(IntEnum):
    WBEM_REFRESH_TYPE_INVALID = 0
    WBEM_REFRESH_TYPE_REMOTE = 3
    WBEM_REFRESH_TYPE_NON_HIPERF = 6


class _WBEM_REFRESH_INFO_REMOTE(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("m_pRefresher", MInterfacePointer(), MInterfacePointer),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRPacketField("m_pTemplate", MInterfacePointer(), MInterfacePointer),
            deferred=True,
        ),
        NDRPacketField("m_Guid", GUID(), GUID),
    ]


class _WBEM_REFRESH_INFO_NON_HIPERF(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("m_wszNamespace", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRPacketField("m_pTemplate", MInterfacePointer(), MInterfacePointer),
            deferred=True,
        ),
    ]


class _WBEM_REFRESH_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRSignedIntField("m_lType", 0),
        NDRUnionField(
            [
                (
                    NDRPacketField(
                        "m_Info", _WBEM_REFRESH_INFO_REMOTE(), _WBEM_REFRESH_INFO_REMOTE
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_lType", None)
                            == WBEM_REFRESH_TYPE.WBEM_REFRESH_TYPE_REMOTE
                        ),
                        (
                            lambda _, val: val.tag
                            == WBEM_REFRESH_TYPE.WBEM_REFRESH_TYPE_REMOTE
                        ),
                    ),
                ),
                (
                    NDRPacketField(
                        "m_Info",
                        _WBEM_REFRESH_INFO_NON_HIPERF(),
                        _WBEM_REFRESH_INFO_NON_HIPERF,
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_lType", None)
                            == WBEM_REFRESH_TYPE.WBEM_REFRESH_TYPE_NON_HIPERF
                        ),
                        (
                            lambda _, val: val.tag
                            == WBEM_REFRESH_TYPE.WBEM_REFRESH_TYPE_NON_HIPERF
                        ),
                    ),
                ),
                (
                    NDRSignedIntField("m_Info", 0),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_lType", None)
                            == WBEM_REFRESH_TYPE.WBEM_REFRESH_TYPE_INVALID
                        ),
                        (
                            lambda _, val: val.tag
                            == WBEM_REFRESH_TYPE.WBEM_REFRESH_TYPE_INVALID
                        ),
                    ),
                ),
            ],
            StrFixedLenField("m_Info", "", length=0),
            align=(4, 8),
            switch_fmt=("l", "l"),
        ),
        NDRSignedIntField("m_lCancelId", 0),
    ]


class AddObjectToRefresher_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pRefresherId", _WBEM_REFRESHER_ID(), _WBEM_REFRESHER_ID),
        NDRConfVarStrNullFieldUtf16("wszPath", ""),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pContext", MInterfacePointer(), MInterfacePointer),
        NDRIntField("dwClientRefrVersion", 0),
    ]


class AddObjectToRefresher_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pInfo", _WBEM_REFRESH_INFO(), _WBEM_REFRESH_INFO),
        NDRIntField("pdwSvrRefrVersion", 0),
        NDRIntField("status", 0),
    ]


class AddObjectToRefresherByTemplate_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pRefresherId", _WBEM_REFRESHER_ID(), _WBEM_REFRESHER_ID),
        NDRPacketField("pTemplate", MInterfacePointer(), MInterfacePointer),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pContext", MInterfacePointer(), MInterfacePointer),
        NDRIntField("dwClientRefrVersion", 0),
    ]


class AddObjectToRefresherByTemplate_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pInfo", _WBEM_REFRESH_INFO(), _WBEM_REFRESH_INFO),
        NDRIntField("pdwSvrRefrVersion", 0),
        NDRIntField("status", 0),
    ]


class AddEnumToRefresher_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pRefresherId", _WBEM_REFRESHER_ID(), _WBEM_REFRESHER_ID),
        NDRConfVarStrNullFieldUtf16("wszClass", ""),
        NDRSignedIntField("lFlags", 0),
        NDRPacketField("pContext", MInterfacePointer(), MInterfacePointer),
        NDRIntField("dwClientRefrVersion", 0),
    ]


class AddEnumToRefresher_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pInfo", _WBEM_REFRESH_INFO(), _WBEM_REFRESH_INFO),
        NDRIntField("pdwSvrRefrVersion", 0),
        NDRIntField("status", 0),
    ]


class RemoveObjectFromRefresher_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pRefresherId", _WBEM_REFRESHER_ID(), _WBEM_REFRESHER_ID),
        NDRSignedIntField("lId", 0),
        NDRSignedIntField("lFlags", 0),
        NDRIntField("dwClientRefrVersion", 0),
    ]


class RemoveObjectFromRefresher_Response(NDRPacket):
    fields_desc = [NDRIntField("pdwSvrRefrVersion", 0), NDRIntField("status", 0)]


class GetRemoteRefresher_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pRefresherId", _WBEM_REFRESHER_ID(), _WBEM_REFRESHER_ID),
        NDRSignedIntField("lFlags", 0),
        NDRIntField("dwClientRefrVersion", 0),
    ]


class GetRemoteRefresher_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppRemRefresher", MInterfacePointer(), MInterfacePointer)
        ),
        NDRPacketField("pGuid", GUID(), GUID),
        NDRIntField("pdwSvrRefrVersion", 0),
        NDRIntField("status", 0),
    ]


class _WBEM_RECONNECT_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRSignedIntField("m_lType", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("m_pwcsPath", ""), deferred=True
        ),
    ]


class _WBEM_RECONNECT_RESULTS(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRSignedIntField("m_lId", 0), NDRSignedIntField("m_hr", 0)]


class ReconnectRemoteRefresher_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pRefresherId", _WBEM_REFRESHER_ID(), _WBEM_REFRESHER_ID),
        NDRSignedIntField("lFlags", 0),
        NDRSignedIntField("lNumObjects", None, size_of="apReconnectResults"),
        NDRIntField("dwClientRefrVersion", 0),
        NDRConfPacketListField(
            "apReconnectInfo",
            [],
            _WBEM_RECONNECT_INFO,
            size_is=lambda pkt: pkt.lNumObjects,
        ),
        NDRConfPacketListField(
            "apReconnectResults",
            [],
            _WBEM_RECONNECT_RESULTS,
            size_is=lambda pkt: pkt.lNumObjects,
        ),
    ]


class ReconnectRemoteRefresher_Response(NDRPacket):
    fields_desc = [
        NDRConfPacketListField(
            "apReconnectResults",
            [],
            _WBEM_RECONNECT_RESULTS,
            size_is=lambda pkt: pkt.lNumObjects,
        ),
        NDRIntField("pdwSvrRefrVersion", 0),
        NDRIntField("status", 0),
    ]


IWBEMREFRESHINGSERVICES_OPNUMS = {
    0: DceRpcOp(AddObjectToRefresher_Request, AddObjectToRefresher_Response),
    1: DceRpcOp(
        AddObjectToRefresherByTemplate_Request, AddObjectToRefresherByTemplate_Response
    ),
    2: DceRpcOp(AddEnumToRefresher_Request, AddEnumToRefresher_Response),
    3: DceRpcOp(RemoveObjectFromRefresher_Request, RemoveObjectFromRefresher_Response),
    4: DceRpcOp(GetRemoteRefresher_Request, GetRemoteRefresher_Response),
    5: DceRpcOp(ReconnectRemoteRefresher_Request, ReconnectRemoteRefresher_Response),
}
register_dcerpc_interface(
    name="IWbemRefreshingServices",
    uuid=uuid.UUID("2C9273E0-1DC3-11d3-B364-00105A1F8177"),
    version="0.0",
    opnums=IWBEMREFRESHINGSERVICES_OPNUMS,
)


class Next_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("proxyGUID", GUID(), GUID),
        NDRSignedIntField("lTimeout", 0),
        NDRIntField("uCount", 0),
    ]


class Next_Response(NDRPacket):
    fields_desc = [
        NDRIntField("puReturned", 0),
        NDRIntField("pdwBuffSize", None, size_of="pBuffer"),
        NDRConfStrLenField("pBuffer", "", size_is=lambda pkt: pkt.pdwBuffSize),
        NDRIntField("status", 0),
    ]


IWBEMWCOSMARTENUM_OPNUMS = {0: DceRpcOp(Next_Request, Next_Response)}
register_com_interface(
    name="IWbemWCOSmartEnum",
    uuid=uuid.UUID("423EC01E-2E35-11d2-B604-00104B703EFD"),
    opnums=IWBEMWCOSMARTENUM_OPNUMS,
)


class GetSmartEnum_Request(NDRPacket):
    fields_desc = []


class GetSmartEnum_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppSmartEnum", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


IWBEMFETCHSMARTENUM_OPNUMS = {0: DceRpcOp(GetSmartEnum_Request, GetSmartEnum_Response)}
register_com_interface(
    name="IWbemFetchSmartEnum",
    uuid=uuid.UUID("1C1C45EE-4395-11d2-B60B-00104B703EFD"),
    opnums=IWBEMFETCHSMARTENUM_OPNUMS,
)
